SystemOrganization addCategory: #TusLibros!

Object subclass: #Cashier
	instanceVariableNames: 'catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:23'!
invalidCheckOut
	^'Invalid check out'! !

!Cashier class methodsFor: 'instance creation' stamp: 'msalegas 5/29/2013 20:50'!
withCatalog: aCatalog 
	^self new catalog: aCatalog! !

!Cashier methodsFor: 'accessing' stamp: 'msalegas 5/29/2013 20:50'!
catalog: aCatalog 
	catalog := aCatalog! !

!Cashier methodsFor: 'services' stamp: 'msalegas 5/29/2013 21:08'!
checkout: aCart creditCard: aCreditCard 
	| price |
	aCart isEmpty ifTrue: [ ^self error: self class invalidCheckOut ].
	aCart do: [:product | 
			price := catalog at: product.
			self debit: price from: aCreditCard.
		]
		! !

!Cashier methodsFor: 'services' stamp: 'msalegas 5/29/2013 21:15'!
debit: anAmount from: aCreditCard 
	 ^aCreditCard debit: anAmount ! !

Object subclass: #CreditCard
	instanceVariableNames: 'balance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'error' stamp: 'msalegas 5/29/2013 21:04'!
notEnoughCredit
	^'Credit card has not enough money'! !

!CreditCard class methodsFor: 'instance creation' stamp: 'msalegas 5/29/2013 20:39'!
withBalance: aNumber
	 ^self new balance: aNumber.! !

!CreditCard methodsFor: 'accessing' stamp: 'msalegas 5/29/2013 20:39'!
balance
	^ balance! !

!CreditCard methodsFor: 'accessing' stamp: 'msalegas 5/29/2013 20:38'!
balance: aNumber
	balance := aNumber! !

!CreditCard methodsFor: 'services' stamp: 'msalegas 5/29/2013 20:54'!
debit: anAmount 
	balance < anAmount ifTrue: [ self error: self class notEnoughCredit ].
	balance := balance - anAmount! !

Bag subclass: #Cart
	instanceVariableNames: 'catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:07'!
productIsNotInTheCatalog
	^'Product not in the catalog'! !

!Cart class methodsFor: 'instance creation' stamp: 'achavez 5/27/2013 21:26'!
withCatalog: aCatalog 
	^self new catalog: aCatalog! !

!Cart methodsFor: 'services' stamp: 'msalegas 5/29/2013 20:37'!
add: aProduct
	(catalog includesKey: aProduct) ifTrue: [super add: aProduct] ifFalse: [ self error: self class productIsNotInTheCatalog]
	! !

!Cart methodsFor: 'accessing' stamp: 'msalegas 5/29/2013 19:33'!
catalog: aCatalog 
	catalog := aCatalog! !

TestCase subclass: #TestCart
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TestCart methodsFor: '1 - Cart' stamp: 'msalegas 5/29/2013 19:41'!
test01CartShouldBeEmptyWhenCreated
	|catalog cart|
	catalog := Set new.
	cart := Cart withCatalog: catalog.
	self assert: cart isEmpty! !

!TestCart methodsFor: '1 - Cart' stamp: 'msalegas 5/29/2013 20:55'!
test02CartShouldListBooksWhenABookIsAdded
	|catalog cart book|
	book := 'Lord of the Rings'.
	catalog := Dictionary new.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.
	cart add: book.
	self assert: (cart includes: book); assert: cart size = 1! !

!TestCart methodsFor: '1 - Cart' stamp: 'msalegas 5/29/2013 21:21'!
test03ABookShouldNotBeAddedToCartIfItIsNotInTheCatalog
	|cart catalog book|
	catalog := Dictionary new.
	cart := Cart withCatalog: catalog.
	book := 'Lord of the Rings'.
	self should: [cart add: book] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cart productIsNotInTheCatalog ]. 
	self assert: cart isEmpty
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'msalegas 5/29/2013 20:56'!
test04AddABookMultipleTimesMakesTheCartSizeToIncrease
	|cart catalog book|
	book := 'Lord of the Rings'.
	catalog := Dictionary new.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.	
	cart add: book; add: book.
	self assert: (cart size = 2)
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'msalegas 5/29/2013 20:57'!
test05CartShouldListAllBooksWhenMultipleBooksAreAdded
	|lotr hobbit catalog cart|
	lotr := 'Lord of the Rings'.
	hobbit := 'The Hobbit'.
	catalog := Dictionary new.
	catalog at: lotr put: 20; at: hobbit put: 10.
	cart := Cart withCatalog: catalog.
	cart add: lotr; add: hobbit.
	self assert: (cart includes: lotr); 
		assert: (cart includes: hobbit);
		assert: cart size = 2.! !

!TestCart methodsFor: '2 - Cashier' stamp: 'msalegas 5/29/2013 21:01'!
test06CheckOutWithEmptyCartRaisesErrorAndCreditCardBalanceIsNotModified
	|cart cashier card prevBalance|
	cart := Cart withCatalog: Set new.
	cashier := Cashier new.
	card := CreditCard withBalance: 20.
	prevBalance := card balance.
	self should: [cashier checkout: cart creditCard: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier invalidCheckOut ].
	 self assert: card balance = prevBalance! !

!TestCart methodsFor: '2 - Cashier' stamp: 'msalegas 5/29/2013 20:51'!
test07CheckOutWithOneProductDebitsMoneyFromCreditCard
	|catalog book cart cashier prevBalance card |
	catalog := Dictionary new.
	book := 'lotr'.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.
	cart add: book.
	cashier := Cashier withCatalog: catalog.
	card := CreditCard withBalance: 100.
	prevBalance := card balance.
	cashier checkout: cart creditCard: card.
	self assert: (card balance = (prevBalance - catalog at: book))
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'msalegas 5/29/2013 21:05'!
test08CheckOutWithInvalidCreditCardDebitsNoMoneyAndRaisesError
	|catalog book cart cashier prevBalance card |
	catalog := Dictionary new.
	book := 'lotr'.
	catalog at: book put: 30.
	cart := Cart withCatalog: catalog.
	cart add: book.
	cashier := Cashier withCatalog: catalog.
	card := CreditCard withBalance: 10.
	prevBalance := card balance.
	self should: [ cashier checkout: cart creditCard: card ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = CreditCard notEnoughCredit ]. 
	self assert: (card balance = prevBalance)
	! !
